@@ -3,10 +3,12 @@ from functools import wraps as functools_wraps
 import inspect
 from enum import Enum
 from pathlib import Path
+import datetime
 from dataclasses import is_dataclass
-from typing import Any
+from weakref import WeakValueDictionary
+from typing import Any, Literal, Union
 from typing_extensions import Annotated, _AnnotatedAlias
-from macrokit import Expr, register_type, symbol, Head
+from macrokit import Expr, register_type, Head
 
 from .gui.class_gui import (
     ClassGuiBase, 
@@ -15,6 +17,7 @@ from .gui.class_gui import (
     MainWindowClassGui,
     SubWindowsClassGui,
     ScrollableClassGui,
+    DraggableClassGui,
     ButtonClassGui, 
     CollapsibleClassGui,
     SplitClassGui,
@@ -25,26 +28,35 @@ from .gui.class_gui import (
     )
 from .gui._base import PopUpMode, ErrorMode, defaults, MagicTemplate, check_override
 from .gui import ContextMenuGui, MenuGui, MenuGuiBase
-from .utils import iter_members
 from ._app import get_app
 
+_datetime = Expr(Head.getattr, [datetime, datetime.datetime])
+_date = Expr(Head.getattr, [datetime, datetime.date])
+_time = Expr(Head.getattr, [datetime, datetime.time])
+
+# magicgui-style input
 register_type(Enum, lambda e: repr(str(e.name)))
 register_type(Path, lambda e: f"r'{e}'")
+register_type(datetime.datetime, lambda e: Expr.parse_call(_datetime, (e.year, e.month, e.day, e.hour, e.minute), {}))
+register_type(datetime.date, lambda e: Expr.parse_call(_date, (e.year, e.month, e.day), {}))
+register_type(datetime.time, lambda e: Expr.parse_call(_time, (e.hour, e.minute), {}))
 
+@register_type(MagicTemplate)
 def find_myname(gui: MagicTemplate):
+    """This function is the essential part of macro recording"""
     parent = gui.__magicclass_parent__
     if parent is None:
         return gui._my_symbol
     else:
         return Expr(Head.getattr, [find_myname(parent), gui._my_symbol])
-
-register_type(MagicTemplate, find_myname)
-
+    
 _BASE_CLASS_SUFFIX = "_Base"
+_POST_INIT = "__post_init__"
 
 class WidgetType(Enum):
     none = "none"
     scrollable = "scrollable"
+    draggalbe = "draggable"
     split = "split"
     collapsible = "collapsible"
     button = "button"
@@ -56,9 +68,20 @@ class WidgetType(Enum):
     groupbox = "groupbox"
     mainwindow = "mainwindow"
 
+WidgetTypeStr = Union[Literal["none"], Literal["scrollable"], Literal["draggable"], Literal["split"],
+                      Literal["collapsible"], Literal["button"], Literal["toolbox"], Literal["tabbed"],
+                      Literal["stacked"], Literal["list"], Literal["subwindows"], Literal["groupbox"],
+                      Literal["mainwindow"]]
+
+PopUpModeStr = Union[Literal["popup"], Literal["first"], Literal["last"], Literal["above"],
+                     Literal["below"], Literal["dock"], Literal["parentlast"]]
+
+ErrorModeStr = Union[Literal["msgbox"], Literal["stderr"]]
+
 _TYPE_MAP = {
     WidgetType.none: ClassGui,
     WidgetType.scrollable: ScrollableClassGui,
+    WidgetType.draggalbe: DraggableClassGui,
     WidgetType.split: SplitClassGui,
     WidgetType.collapsible: CollapsibleClassGui,
     WidgetType.button: ButtonClassGui,
@@ -93,15 +116,15 @@ def Bound(value: Any) -> _AnnotatedAlias:
     
     return Annotated[Any, {"bind": value}]
 
-def magicclass(class_: type|None = None,
+def magicclass(class_: type | None = None,
                *,
                layout: str = "vertical", 
                labels: bool = True, 
                name: str = None,
                close_on_run: bool = None,
-               popup_mode: str | PopUpMode = None,
-               error_mode: str | ErrorMode = None,
-               widget_type: str | WidgetType = WidgetType.none,
+               popup_mode: PopUpModeStr | PopUpMode = None,
+               error_mode: ErrorModeStr | ErrorMode = None,
+               widget_type: WidgetTypeStr | WidgetType = WidgetType.none,
                parent = None
                ):
     """
@@ -164,9 +187,8 @@ def magicclass(class_: type|None = None,
         class_gui = _TYPE_MAP[widget_type]
         
         if not issubclass(cls, MagicTemplate):
-            _check_collision(cls, class_gui)
             check_override(cls)
-            
+        
         # get class attributes first
         doc = cls.__doc__
         sig = inspect.signature(cls)
@@ -202,15 +224,17 @@ def magicclass(class_: type|None = None,
             super(oldclass, self).__init__(*args, **kwargs)
             self._convert_attributes_into_widgets()
             
-            if hasattr(self, "__post_init__"):
+            if hasattr(self, _POST_INIT):
                 self.__post_init__()
             
-            # Record class instance construction
-            macro_init = Expr.parse_init(self, cls, args, kwargs)
-            self._recorded_macro.append(macro_init)
-            
             if widget_type in (WidgetType.collapsible, WidgetType.button):
                 self.btn_text = self.name
+            
+            # TODO: does this "if" make sense?
+            if self.__magicclass_parent__ is None:
+                macrowidget = self.macro.widget.native
+                macrowidget.setParent(self.native, macrowidget.windowFlags())
+                self.macro.widget.__magicclass_parent__ = self
 
         newclass.__init__ = __init__
         
@@ -337,9 +361,8 @@ def _call_magicmenu(class_: type = None,
             raise TypeError("dataclass is not compatible with magicclass.")
         
         if not issubclass(cls, MagicTemplate):
-            _check_collision(cls, menugui_class)
             check_override(cls)
-        
+            
         # get class attributes first
         doc = cls.__doc__
         sig = inspect.signature(cls)
@@ -368,12 +391,8 @@ def _call_magicmenu(class_: type = None,
             super(oldclass, self).__init__(*args, **kwargs)
             self._convert_attributes_into_widgets()
             
-            if hasattr(self, "__post_init__"):
+            if hasattr(self, _POST_INIT):
                 self.__post_init__()
-            
-            # Record class instance construction
-            macro_init = Expr.parse_init(self, cls, args, kwargs)
-            self._recorded_macro.append(macro_init)
 
         newclass.__init__ = __init__
         
@@ -387,16 +406,33 @@ def _call_magicmenu(class_: type = None,
 magicmenu.__doc__ += _call_magicmenu.__doc__
 magiccontext.__doc__ += _call_magicmenu.__doc__
 
+_HELPS: WeakValueDictionary[int, MagicTemplate] = {}
 
-def _check_collision(cls0: type, cls1: type):
+def build_help(ui: MagicTemplate, parent=None):
     """
-    Check if two classes have name collisions.
+    Build a widget for user guide. Once it is built, widget will be cached.
+
+    Parameters
+    ----------
+    ui : MagicTemplate
+        Magic class UI object.
+
+    Returns
+    -------
+    HelpWidget
+        Help of the input UI.
     """    
-    mem0 = set(x[0] for x in iter_members(cls0))
-    mem1 = set(x[0] for x in iter_members(cls1))
-    collision = mem0 & mem1
-    if collision:
-        raise AttributeError(f"Collision between {cls0.__name__} and {cls1.__name__}: {collision}")
+    ui_id = id(ui)
+    if ui_id in _HELPS.keys():
+        help_widget = _HELPS[ui_id]
+    else:
+        from .help import HelpWidget
+        if parent is None:
+            parent = ui.native
+        help_widget = HelpWidget(ui, parent=parent)
+        _HELPS[ui_id] = help_widget
+    return help_widget
+        
 
 class _CallableClass:
     def __init__(self):
@@ -450,5 +486,3 @@ class Parameters(_CallableClass):
         """        
         params = list(self.__signature__.parameters.keys())[1:]
         return {param: getattr(self, param) for param in params}
-    
-    
\ No newline at end of file
