@@ -6,24 +6,27 @@ import inspect
 from enum import Enum
 import warnings
 from collections.abc import MutableSequence
+from docstring_parser import parse, compose
 
 from magicgui.events import Signal
 from magicgui.signature import MagicParameter
 from magicgui.widgets import FunctionGui, FileEdit, EmptyWidget, Widget, Container
 from magicgui.widgets._bases import ValueWidget
-from macrokit import Macro, Expr, Head, Symbol, symbol
+from macrokit import Expr, Head, Symbol, symbol
 
 from .keybinding import as_shortcut
-from .mgui_ext import AbstractAction, FunctionGuiPlus, PushButtonPlus
+from .mgui_ext import AbstractAction, Action, FunctionGuiPlus, PushButtonPlus
 from .utils import get_parameters
+from ._macro import GuiMacro
 
 from ..utils import get_signature, iter_members, extract_tooltip, screen_center
-from ..widgets import Separator, MacroEdit
+from ..widgets import Separator
 from ..fields import MagicField
 from ..signature import MagicMethodSignature, get_additional_option
 from ..wrappers import upgrade_signature
 
 if TYPE_CHECKING:
+    import numpy as np
     import napari
     from qtpy.QtWidgets import QDockWidget
 
@@ -46,29 +49,29 @@ defaults = {"popup_mode": PopUpMode.popup,
             }
 
 _RESERVED = {"__magicclass_parent__", "__magicclass_children__", "_close_on_run", 
-             "_error_mode", "_popup_mode", "_recorded_macro", "_my_symbol", "_macro_instance", 
+             "_error_mode", "_popup_mode", "_my_symbol", "_macro_instance", "macro",
              "annotation", "enabled", "gui_only", "height", "label_changed", "label", "layout",
              "labels", "margins", "max_height", "max_width", "min_height", "min_width", "name",
              "options", "param_kind", "parent_changed", "tooltip", "visible", "widget_type", 
-             "width", "parent_viewer", "parent_dock_widget", "objectName", "create_macro", "wraps",
-             "_unwrap_method", "_search_parent_magicclass", "_iter_child_magicclasses",
+             "width", "wraps", "_unwrap_method", "_search_parent_magicclass", 
+             "_iter_child_magicclasses", 
              }
 
-UI = Symbol("ui")
-
 def check_override(cls: type):
     subclass_members = set(cls.__dict__.keys())
     collision = subclass_members & _RESERVED
     if collision:
         raise AttributeError(f"Cannot override magic class reserved attributes: {collision}")
+          
 
-class MagicTemplate:    
+class MagicTemplate: 
+    __doc__ = ""
     __magicclass_parent__: None | MagicTemplate
     __magicclass_children__: list[MagicTemplate]
     _close_on_run: bool
-    _component_class: type[AbstractAction | Widget]
+    _component_class: type[Action | Widget]
     _error_mode: ErrorMode
-    _macro_instance: Macro
+    _macro_instance: GuiMacro
     _my_symbol: Symbol
     _popup_mode: PopUpMode
     annotation: Any
@@ -127,18 +130,22 @@ class MagicTemplate:
     def insert(self, key: int, widget: Widget) -> None:
         raise NotImplementedError()
     
+    def render(self) -> "np.ndarray":
+        raise NotImplementedError()
+    
     @property
-    def _recorded_macro(self):
+    def macro(self) -> GuiMacro:
         if self.__magicclass_parent__ is None:
             return self._macro_instance
         else:
-            return self.__magicclass_parent__._recorded_macro
+            return self.__magicclass_parent__.macro
     
     @property
     def parent_viewer(self) -> "napari.Viewer" | None:
         """
         Return napari.Viewer if magic class is a dock widget of a viewer.
         """
+        # TODO: https://github.com/napari/napari/pull/3748
         parent_self = self._search_parent_magicclass()
         try:
             viewer = parent_self.parent.parent().qt_viewer.viewer
@@ -167,24 +174,15 @@ class MagicTemplate:
         return self.native.objectName()
     
     def create_macro(self, show: bool = False) -> str:
-        """
-        Create executable Python scripts from the recorded macro object.
-
-        Parameters
-        ----------
-        show : bool, default is False
-            Launch a TextEdit window that shows recorded macro.
-        """
+        msg = "Method 'create_macro' is deprecated and will be removed soon."\
+              "Macro object is available via 'self.macro' property, and the widget is"\
+              "available at 'self.macro.widget'."
+        warnings.warn(msg, DeprecationWarning)
         
-        out = str(self._recorded_macro)
+        out = str(self.macro)
                     
         if show:
-            win = MacroEdit(name="macro")
-            win.value = out
-            
-            win.native.setParent(self.native, win.native.windowFlags())
-            win.native.move(screen_center() - win.native.rect().center())
-            win.show()
+            self.macro.widget.show()
             
         return out
     
@@ -200,13 +198,14 @@ class MagicTemplate:
         Basically, this function is used as a wrapper like below.
         
         .. code-block:: python
-        @magicclass
-        class C:
+        
             @magicclass
-            class D: 
-                def func(self, ...): ... # pre-definition
-            @D.wraps
-            def func(self, ...): ...
+            class C:
+                @magicclass
+                class D: 
+                    def func(self, ...): ... # pre-definition
+                @D.wraps
+                def func(self, ...): ...
 
         Parameters
         ----------
@@ -286,6 +285,8 @@ class MagicTemplate:
                         child_widget.changed.disconnect()
                         child_widget.changed.connect(widget.changed)
                         child_widget.tooltip = widget.tooltip
+                        child_widget._doc = widget._doc
+                widget._unwrapped = True
                 break
         else:
             raise RuntimeError(f"{child_clsname} not found in class {self.__class__.__name__}")
@@ -327,7 +328,7 @@ class MagicTemplate:
         if _need_record(obj):
             @functools_wraps(obj)
             def func(*args, **kwargs):
-                with self._recorded_macro.blocked():
+                with self.macro.blocked():
                     out = _inner_func(*args, **kwargs)
                 return out
         else:
@@ -341,6 +342,7 @@ class MagicTemplate:
         
         # Prepare a button or action
         widget.tooltip = extract_tooltip(func)
+        widget._doc = func.__doc__
         
         # Get the number of parameters except for empty widgets.
         # With these lines, "bind" method of magicgui works inside magicclass.
@@ -352,6 +354,7 @@ class MagicTemplate:
         all_params = []
         for param in func.__signature__.parameters.values():
             if isinstance(param.annotation, _AnnotatedAlias):
+                # TODO: pydantic
                 param = MagicParameter.from_parameter(param)
             if isinstance(param, MagicParameter):
                 bound_value = param.options.get("bind", None)
@@ -378,7 +381,9 @@ class MagicTemplate:
         if isinstance(func.__signature__, MagicMethodSignature):
             # NOTE: I don't know the reason why "additional_options" is lost. 
             func.__signature__.additional_options = getattr(obj_sig, "additional_options", {})
-            
+        
+        # TODO: "update_widget" argument is useful.
+        # see https://github.com/napari/magicgui/pull/309
         if nparams == 0:
             # We don't want a dialog with a single widget "Run" to show up.
             def run_function():
@@ -525,13 +530,13 @@ class MagicTemplate:
     
 class BaseGui(MagicTemplate):
     def __init__(self, close_on_run, popup_mode, error_mode):
-        self._macro_instance = Macro(flags={"Get": False, "Return": False})
+        self._macro_instance = GuiMacro(flags={"Get": False, "Return": False})
         self.__magicclass_parent__: None | BaseGui = None
         self.__magicclass_children__: list[MagicTemplate] = []
         self._close_on_run = close_on_run
         self._popup_mode = popup_mode
         self._error_mode = error_mode
-        self._my_symbol = UI
+        self._my_symbol = Symbol.var("ui")
 
 def _get_widget_name(widget: Widget):
     # To escape reference
@@ -543,10 +548,11 @@ def _temporal_function_gui_callback(bgui: MagicTemplate, fgui: FunctionGuiPlus,
     else:
         raise TypeError("fgui must be FunctionGui object.")
         
+    return_type = fgui.return_annotation
+    result_required = return_type is not inspect._empty
+    
     def _after_run():
         bound = fgui._previous_bound
-        return_type = fgui.return_annotation
-        result_required = return_type is not inspect._empty
         result = Symbol("result")
         # Standard button will be connected with two callbacks.
         # 1. Build FunctionGui
@@ -558,13 +564,13 @@ def _temporal_function_gui_callback(bgui: MagicTemplate, fgui: FunctionGuiPlus,
             line = Expr(head=Head.call, args=[ev])
             if result_required:
                 line = Expr(head=Head.assign, args=[result, line])
-            bgui._recorded_macro.append(line)
+            bgui.macro.append(line)
         else:
             kwargs = {k: v for k, v in bound.arguments.items()}
             line = Expr.parse_method(bgui, _function, (), kwargs)
             if result_required:
                 line = Expr(Head.assign, [result, line])
-            bgui._recorded_macro.append(line)
+            bgui.macro.append(line)
         
         # Deal with return annotation
         
@@ -575,8 +581,8 @@ def _temporal_function_gui_callback(bgui: MagicTemplate, fgui: FunctionGuiPlus,
                 b = Expr(head=Head.getitem, args=[symbol(bgui), widget.name])
                 _gui = Expr(head=Head.getattr, args=[b, Symbol("mgui")])
                 line = Expr.parse_call(callback, (_gui, result, return_type), {})
-                bgui._recorded_macro.append(line)
-        
+                bgui.macro.append(line)
+        bgui.macro._last_setval = None
         return None
     
     return _after_run
@@ -657,6 +663,14 @@ def wraps(template: Callable | inspect.Signature) -> Callable[[_C], _C]:
             parameters=new_params,
             return_annotation=return_annotation
             )
+        
+        fdoc = parse(f.__doc__)
+        tempdoc = parse(template.__doc__)
+        fdoc.short_description = fdoc.short_description or tempdoc.short_description
+        fdoc.long_description = fdoc.long_description or tempdoc.long_description
+        fdoc.meta = fdoc.meta or tempdoc.meta
+        f.__doc__ = compose(fdoc)
+        
         return f
     return wrapper
 
@@ -746,10 +760,13 @@ def _need_record(func: Callable):
     return get_additional_option(func, "record", True)
 
 def value_widget_callback(gui: MagicTemplate, widget: ValueWidget, name: str, getvalue: bool = True):
+    """
+    Define a ValueWidget callback, including macro recording.
+    """    
     sym_name = Symbol(name)
     sym_value = Symbol("value")
     def _set_value():
-        if not widget.enabled:
+        if not widget.enabled or not gui.macro.active:
             # If widget is read only, it means that value is set in script (not manually).
             # Thus this event should not be recorded as a macro.
             return None
@@ -759,31 +776,34 @@ def value_widget_callback(gui: MagicTemplate, widget: ValueWidget, name: str, ge
         if getvalue:
             sub = Expr(head=Head.getattr, args=[sym_name, sym_value]) # name.value
         else:
-            sub = Expr(head=Head.value, args=[sym_name])
+            sub = sym_name
         
         # Make an expression of
         # >>> x.name.value = value
         # or
         # >>> x.name = value
-        expr = Expr(head=Head.assign, 
-                    args=[Expr(head=Head.getattr, 
-                               args=[symbol(gui), sub]), 
-                          widget.value])
+        target = Expr(Head.getattr, [symbol(gui), sub])
+        expr = Expr(Head.assign, [target, widget.value])
         
-        last_expr = gui._recorded_macro[-1]
-        if (len(gui._recorded_macro) > 1 and 
-            last_expr.head == expr.head and 
-            last_expr.args[0].args[1].head == expr.args[0].args[1].head and
-            last_expr.args[0].args[1].args[0] == expr.args[0].args[1].args[0]):
-            gui._recorded_macro[-1] = expr
+        if gui.macro._last_setval == target and len(gui.macro) > 0:
+            gui.macro.pop()
+            gui.macro._erase_last()
         else:
-            gui._recorded_macro.append(expr)
+            gui.macro._last_setval = target
+        gui.macro.append(expr)
         return None
     return _set_value
 
 def nested_function_gui_callback(gui: MagicTemplate, fgui: FunctionGui):
+    """
+    Define a FunctionGui callback, including macro recording.
+    """    
     fgui_name = Symbol(fgui.name)
     def _after_run():
+        if not fgui.enabled or not gui.macro.active:
+            # If widget is read only, it means that value is set in script (not manually).
+            # Thus this event should not be recorded as a macro.
+            return None
         inputs = get_parameters(fgui)
         args = [Expr(head=Head.kw, args=[Symbol(k), v]) for k, v in inputs.items()]
         # args[0] is self
@@ -793,10 +813,13 @@ def nested_function_gui_callback(gui: MagicTemplate, fgui: FunctionGui):
         if fgui._auto_call:
             # Auto-call will cause many redundant macros. To avoid this, only the last input
             # will be recorded in magic-class.
-            last_expr = gui._recorded_macro[-1]
-            if last_expr.head == Head.call and last_expr.args[0].head == Head.getattr and \
-                last_expr.args[0].args[1] == expr.args[0].args[1]:
-                gui._recorded_macro.pop()
+            last_expr = gui.macro[-1]
+            if (last_expr.head == Head.call and last_expr.args[0].head == Head.getattr and
+                last_expr.args[0].args[1] == expr.args[0].args[1] and
+                len(gui.macro) > 0):
+                gui.macro.pop()
+                gui.macro._erase_last()
 
-        gui._recorded_macro.append(expr)
+        gui.macro.append(expr)
+        gui.macro._last_setval = None
     return _after_run
\ No newline at end of file
