@@ -12,21 +12,22 @@ from .mgui_ext import PushButtonPlus
 from .menu_gui import MenuGui, ContextMenuGui
 from ._base import BaseGui, PopUpMode, ErrorMode, value_widget_callback, nested_function_gui_callback
 from .utils import define_callback, MagicClassConstructionError
-from ._containers import (
+from ..widgets import (
     ButtonContainer,
     GroupBoxContainer,
     ListContainer,
     SubWindowsContainer,
     ScrollableContainer,
+    DraggableContainer,
     CollapsibleContainer,
     SplitterContainer,
     StackedContainer,
     TabbedContainer,
-    ToolBoxContainer
+    ToolBoxContainer,
+    FreeWidget
     )
 
 from ..utils import iter_members, extract_tooltip
-from ..widgets import FreeWidget
 from ..fields import MagicField
 from ..signature import get_additional_option
 
@@ -245,7 +246,7 @@ def make_gui(container: type[ContainerWidget], no_margin: bool = True):
             _hide_labels = (_LabeledWidget, ButtonWidget, ClassGuiBase, FreeWidget, Label,
                             Image, Table, FunctionGui)
                 
-            if isinstance(widget, ValueWidget):
+            if isinstance(widget, (ValueWidget, ContainerWidget)):
                 widget.changed.connect(lambda: self.changed.emit(self))
             
             if hasattr(widget, "__magicclass_parent__") or \
@@ -327,6 +328,9 @@ class SplitClassGui: pass
 @make_gui(ScrollableContainer)
 class ScrollableClassGui: pass
 
+@make_gui(DraggableContainer)
+class DraggableClassGui: pass
+
 @make_gui(CollapsibleContainer)
 class CollapsibleClassGui: pass
 
