@@ -5,9 +5,9 @@ from qtpy.QtWidgets import QPushButton, QAction, QWidgetAction
 from qtpy.QtGui import QIcon
 from qtpy.QtCore import QSize
 from magicgui.events import Signal
-from magicgui.widgets import Container, PushButton, FunctionGui
+from magicgui.widgets import PushButton, FunctionGui
 from magicgui.widgets._concrete import _LabeledWidget
-from magicgui.widgets._bases import Widget, ValueWidget, ButtonWidget
+from magicgui.widgets._bases import Widget, ValueWidget, ButtonWidget, ContainerWidget
 from magicgui.widgets._function_gui import _function_name_pointing_to_widget
 from matplotlib.colors import to_rgb
 from ..widgets import Separator
@@ -73,7 +73,7 @@ class FunctionGuiPlus(FunctionGui):
     
     def insert(self, key: int, widget: Widget):
         """Insert widget at ``key``."""
-        if isinstance(widget, ValueWidget):
+        if isinstance(widget, (ValueWidget, ContainerWidget)):
             widget.changed.connect(lambda: self.changed.emit(self))
         _widget = widget
 
@@ -100,7 +100,9 @@ class PushButtonPlus(PushButton):
         super().__init__(text=text, **kwargs)
         self.native: QPushButton
         self._icon_path = None
-        self.mgui: Container = None # tagged function GUI
+        self.mgui: FunctionGuiPlus = None # tagged function GUI
+        self._doc = ""
+        self._unwrapped = False
     
     @property
     def running(self) -> bool:
@@ -250,6 +252,8 @@ class Action(AbstractAction):
     def __init__(self, *args, name: str = None, text: str = None, gui_only: bool = True, **kwargs):
         self.native = QAction(*args, **kwargs)
         self.mgui: FunctionGuiPlus = None
+        self._doc = ""
+        self._unwrapped = False
         
         self._icon_path = None
         if text:
